import { lookup, Lookup } from "geoip-lite";
import { getPreferredLanguage } from "../src";

jest.mock("geoip-lite", () => ({
  lookup: jest.fn(),
}));

describe("getPreferredLanguage", () => {
  it("should return the language from the user IP address if local storage is not available", async () => {
    // Mock localStorage
    localStorage.setItem("language", "vi");

    const preferredLanguage = await getPreferredLanguage();
    expect(preferredLanguage).toEqual("vi");

    localStorage.removeItem("language");
  });

  it("should return the language from the user IP address if local storage is not available", async () => {
    // Mock fetch API
    const fetchMock = jest.fn().mockImplementation(
      (): Promise<Response> =>
        Promise.resolve({
          ok: true,
          json: () => Promise.resolve({ ip: "123.45.67.89" }),
        } as Response) // Type assertion to satisfy TypeScript's type checker
    );

    global.fetch = fetchMock;

    // Mock geoip lookup

    const expectedLocation: Lookup = {
      range: [0, 1],
      country: "US",
      region: "CA",
      city: "San Francisco",
      ll: [37.7749, -122.4194],
      metro: 807,
      eu: "0",
      timezone: "America/Los_Angeles",
      area: 123,
    };

    // Set up the mock implementation
    const mockLookup = lookup as jest.MockedFunction<typeof lookup>;
    mockLookup.mockReturnValueOnce(expectedLocation);

    const preferredLanguage = await getPreferredLanguage();
    expect(preferredLanguage).toEqual("en");
  });

  it("should return the default language if both local storage and user IP are not available", async () => {
    // Mock localStorage
    const localStorageMock = {
      getItem: jest.fn().mockReturnValueOnce(null),
    };

    Object.defineProperty(window, "localStorage", {
      value: localStorageMock,
    });

    // Mock fetch API
    global.fetch = jest
      .fn()
      .mockRejectedValueOnce(new Error("Failed to fetch"));

    const preferredLanguage = await getPreferredLanguage();
    expect(preferredLanguage).toEqual("en");
  });
});
