import { lookup } from "geoip-lite";

// Type for valid language codes
type LanguageCode = "vi" | "en";

// Default language to use if no other language is found
const DEFAULT_LANGUAGE: LanguageCode = "en";

// This function gets the language from local storage, if it exists
function getLanguageFromStorage(): LanguageCode | null {
  // Check if we're running in a browser
  if (typeof window === "undefined") {
    return null;
  }
  // Get the language from local storage
  const storageLanguage = localStorage.getItem("language");

  // Check if the language code is valid
  if (storageLanguage === "vi" || storageLanguage === "en") {
    return storageLanguage as LanguageCode;
  }
  // Return null if the language code is invalid or doesn't exist
  return null;
}

// This function gets the language from the user's IP address, if possible
function getLanguageFromIP(ip: string): LanguageCode | null {
  // Use geoip-lite to get the user's country code from their IP address
  const countryCode = lookup(ip)?.country;

  // Map the country code to a language code
  if (countryCode === "VN") {
    return "vi";
  }
  // Return the default language if the country code is unknown or not supported
  return DEFAULT_LANGUAGE;
}

// This function gets the user's IP address from the browser
async function getUserIP(): Promise<string | null> {
  // Check if we're running in a browser
  if (typeof window === "undefined") {
    return null;
  }
  try {
    // Use a free API to get the user's IP address
    const response = await fetch("https://api.ipify.org?format=json");
    const data: any = await response.json();
    return data.ip;
  } catch (error) {
    console.error(error);
    return null;
  }
}

// This function sets the language code in local storage
function setLanguageToStorage(languageCode: LanguageCode) {
  // Check if we're running in a browser
  if (typeof window === "undefined") {
    return;
  }
  // Set the language in local storage
  localStorage.setItem("language", languageCode);
}

// This function gets the user's preferred language, using local storage if available, or falling back to the user's IP address
async function getPreferredLanguage(): Promise<LanguageCode> {
  // Try to get the language from local storage
  const languageFromStorage = getLanguageFromStorage();

  if (languageFromStorage) {
    return languageFromStorage;
  }

  // Try to get the language from the user's IP address
  const userIP = await getUserIP();

  if (userIP) {
    const languageFromIP = getLanguageFromIP(userIP);
    if (languageFromIP) {
      // Set the language in local storage for future use
      setLanguageToStorage(languageFromIP);
      return languageFromIP;
    }
  }

  // Return the default language if all else fails
  return DEFAULT_LANGUAGE;
}

export { getPreferredLanguage };
